from cx_Freeze import setup, Executable
from rpn.rpnVersion import PROGRAM_VERSION
import rpn.rpnGlobals as g

import os

# Dependencies are automatically detected, but it might need fine tuning.
buildOptions = \
    dict(
        packages = [
            'pkg_resources._vendor',
        ],

        include_files = [
            'rpn' + os.sep + g.dataDir + os.sep + 'units.pckl.bz2',
            'rpn' + os.sep + g.dataDir + os.sep + 'unit_conversions.pckl.bz2',
            'rpn' + os.sep + g.dataDir + os.sep + 'unit_help.pckl.bz2',
            'rpn' + os.sep + g.dataDir + os.sep + 'unit_names.pckl.bz2',
            'rpn' + os.sep + g.dataDir + os.sep + 'deltat.data',
            'rpn' + os.sep + g.dataDir + os.sep + 'deltat.preds',
            'rpn' + os.sep + g.dataDir + os.sep + 'Leap_Second.dat',
            'rpn' + os.sep + g.dataDir + os.sep + 'balanced_primes.cache',
            'rpn' + os.sep + g.dataDir + os.sep + 'cousin_primes.cache',
            'rpn' + os.sep + g.dataDir + os.sep + 'double_balanced_primes.cache',
            'rpn' + os.sep + g.dataDir + os.sep + 'huge_primes.cache',
            'rpn' + os.sep + g.dataDir + os.sep + 'isolated_primes.cache',
            'rpn' + os.sep + g.dataDir + os.sep + 'large_primes.cache',
            'rpn' + os.sep + g.dataDir + os.sep + 'quadruple_balanced_primes.cache',
            'rpn' + os.sep + g.dataDir + os.sep + 'quad_primes.cache',
            'rpn' + os.sep + g.dataDir + os.sep + 'quint_primes.cache',
            'rpn' + os.sep + g.dataDir + os.sep + 'sext_primes.cache',
            'rpn' + os.sep + g.dataDir + os.sep + 'sexy_primes.cache',
            'rpn' + os.sep + g.dataDir + os.sep + 'sexy_quadruplets.cache',
            'rpn' + os.sep + g.dataDir + os.sep + 'sexy_triplets.cache',
            'rpn' + os.sep + g.dataDir + os.sep + 'small_primes.cache',
            'rpn' + os.sep + g.dataDir + os.sep + 'sophie_primes.cache',
            'rpn' + os.sep + g.dataDir + os.sep + 'super_primes.cache',
            'rpn' + os.sep + g.dataDir + os.sep + 'triplet_primes.cache',
            'rpn' + os.sep + g.dataDir + os.sep + 'triple_balanced_primes.cache',
            'rpn' + os.sep + g.dataDir + os.sep + 'twin_primes.cache',
        ],

        include_msvcr = 1,
        optimize = 2,
    )

executables = [
    Executable( script = 'rpn.py',
                base = None,
                icon = 'rpn.ico',
                initScript = None,
                shortcutName = 'rpnChilada ' + PROGRAM_VERSION,
                targetName = 'rpn.exe'
    ),

    Executable( script = 'preparePrimeData.py',
                base = None,
                icon = 'rpn.ico',
                initScript = None,
                shortcutName = 'rpnChilada Prepare Prime Data ' + PROGRAM_VERSION,
                targetName = 'rpnPreparePrimeData.exe'
    ),

    Executable( script = 'makeHelp.py',
                base = None,
                icon = 'rpn.ico',
                initScript = None,
                shortcutName = 'rpnChilada Make Help ' + PROGRAM_VERSION,
                targetName = 'rpnMakeHelp.exe'
    ),

    Executable( script = 'makeUnits.py',
                base = None,
                icon = 'rpn.ico',
                initScript = None,
                shortcutName = 'rpnChilada Make Units ' + PROGRAM_VERSION,
                targetName = 'rpnMakeUnits.exe'
    ),

    Executable( script = 'testRPN.py',
                base = None,
                icon = 'rpn.ico',
                initScript = None,
                shortcutName = 'rpnChilada Test ' + PROGRAM_VERSION,
                targetName = 'testRPN.exe'
    )




]

setup( name = 'rpn',
       version = PROGRAM_VERSION,
       author = 'Rick Gutleber',
       author_email = 'rickg@his.com',
       description = 'command-line RPN calculator',
       options = dict( build_exe = buildOptions ),
       executables = executables
)




#!/usr/bin/env python

#******************************************************************************
#
#  rpnAliases.py
#
#  rpnChilada alias declarations
#  copyright (c) 2020, Rick Gutleber (rickg@his.com)
#
#  License: GNU GPL 3.0 (see <http://www.gnu.org/licenses/gpl.html> for more
#  information).
#
#******************************************************************************

import rpn.rpnGlobals as g


#******************************************************************************
#
#  dumpAliases
#
#******************************************************************************

def dumpAliases( ):
    for alias in sorted( g.aliases.keys( ) ):
        print( alias, g.aliases[ alias ] )

    print( )

    return len( g.aliases )


#******************************************************************************
#
#  operatorAliases
#
#******************************************************************************

operatorAliases = {
    '!!'                            : 'double_factorial',
    '!'                             : 'factorial',
    '!='                            : 'is_not_equal',
    '%'                             : 'modulo',
    '*'                             : 'multiply',
    '**'                            : 'power',
    '***'                           : 'tetrate',
    '+'                             : 'add',
    '++'                            : 'increment',
    '-'                             : 'subtract',
    '--'                            : 'decrement',
    '-c'                            : 'comma_mode',
    '-i'                            : 'identify_mode',
    '-o'                            : 'octal_mode',
    '-t'                            : 'timer_mode',
    '-x'                            : 'hex_mode',
    '-z'                            : 'leading_zero_mode',
    '/'                             : 'divide',
    '//'                            : 'root',
    '///'                           : 'super_root',
    '1/x'                           : 'reciprocal',
    '2-nacci'                       : 'fibonacci',
    '3-nacci'                       : 'tribonacci',
    '4-nacci'                       : 'tetranacci',
    '5-nacci'                       : 'pentanacci',
    '6-nacci'                       : 'hexanacci',
    '7-nacci'                       : 'heptanacci',
    '8-nacci'                       : 'octanacci',
    '<'                             : 'is_less',
    '<<'                            : 'shift_left',
    '<='                            : 'is_not_greater',
    '='                             : 'set_variable',
    '=='                            : 'is_equal',
    '>'                             : 'is_greater',
    '>='                            : 'is_not_less',
    '>>'                            : 'shift_right',
    '?'                             : 'help',
    'TP'                            : 'planck_temperature',
    '^'                             : 'power',
    '_aliases'                      : '_dump_aliases',
    '_cache'                        : '_dump_cache',
    '_constants'                    : '_dump_constants',
    '_conversions'                  : '_dump_conversions',
    '_dump_ops'                     : '_dump_operators',
    '_operators'                    : '_dump_operators',
    '_ops'                          : '_dump_operators',
    '_primes'                       : '_dump_prime_cache',
    '_stats'                        : '_dump_stats',
    '_units'                        : '_dump_units',
    'a0'                            : 'fine_structure_constant',
    'ackermann'                     : 'ackermann_number',
    'add_dig'                       : 'add_digits',
    'add_digit'                     : 'add_digits',
    'add_poly'                      : 'add_polynomials',
    'alt_fac'                       : 'alternating_factorial',
    'alt_sign'                      : 'alternate_signs',
    'alt_sign2'                     : 'alternate_signs_2',
    'alt_sign_2'                    : 'alternate_signs_2',
    'alt_sum'                       : 'alternating_sum',
    'alt_sum2'                      : 'alternating_sum_2',
    'alt_sum_2'                     : 'alternating_sum_2',
    'alt_zeta'                      : 'eta',
    'alternating_signs'             : 'alternate_signs',
    'alternating_signs_2'           : 'alternate_signs_2',
    'altfac'                        : 'alternating_factorial',
    'altsign'                       : 'alternate_signs',
    'altsign2'                      : 'alternate_signs_2',
    'altsum'                        : 'alternating_sum',
    'altsum2'                       : 'alternating_sum_2',
    'altzeta'                       : 'eta',
    'apery'                         : 'apery_constant',
    'aperynum'                      : 'nth_apery',
    'arccos'                        : 'acos',
    'arccosecant'                   : 'acsc',
    'arccot'                        : 'acot',
    'arcos'                         : 'acos',
    'arcosine'                      : 'acos',
    'arcot'                         : 'acot',
    'arcotangent'                   : 'acot',
    'arcsec'                        : 'asec',
    'arcsecant'                     : 'asec',
    'arcsin'                        : 'asin',
    'arcsine'                       : 'asin',
    'arctan'                        : 'atan',
    'arctangent'                    : 'atan',
    'arg'                           : 'argument',
    'arithmean'                     : 'mean',
    'arithmetic_mean'               : 'mean',
    'ascension_thursday'            : 'ascension',
    'ash_wed'                       : 'ash_wednesday',
    'ashwed'                        : 'ash_wednesday',
    'ashwednesday'                  : 'ash_wednesday',
    'astro_dawn'                    : 'astronomical_dawn',
    'astro_dusk'                    : 'astronomical_dusk',
    'autumn'                        : 'autumnal_equinox',
    'autumnal'                      : 'autumnal_equinox',
    'average'                       : 'mean',
    'avg'                           : 'mean',
    'bahai'                         : 'to_bahai',
    'bahai_name'                    : 'to_bahai_name',
    'bal'                           : 'balanced_prime',
    'bal_'                          : 'balanced_primes',
    'balanced'                      : 'balanced_prime',
    'balanced_'                     : 'balanced_primes',
    'balanced_prime_'               : 'balanced_primes',
    'base_unit'                     : 'base_units',
    'bell'                          : 'nth_bell',
    'bell_poly'                     : 'bell_polynomial',
    'bellpoly'                      : 'bell_polynomial',
    'bernoulli'                     : 'nth_bernoulli',
    'bicron'                        : 'picometer',
    'bit_count'                     : 'count_bits',
    'bits'                          : 'count_bits',
    'boiling_point'                 : 'element_boiling_point',
    'c_cube'                        : 'centered_cube',
    'c_cubic'                       : 'centered_cube',
    'c_dec'                         : 'centered_decagonal',
    'c_dec?'                        : 'nth_centered_decagonal',
    'c_decagonal'                   : 'centered_decagonal',
    'c_decagonal?'                  : 'nth_centered_decagonal',
    'c_dodeca'                      : 'centered_dodecahedral',
    'c_dodecahedral'                : 'centered_dodecahedral',
    'c_hept'                        : 'centered_heptagonal',
    'c_hept?'                       : 'nth_centered_heptagonal',
    'c_heptagonal'                  : 'centered_heptagonal',
    'c_heptagonal?'                 : 'nth_centered_heptagonal',
    'c_hexagonal'                   : 'centered_hexagonal',
    'c_hexagonal?'                  : 'nth_centered_hexagonal',
    'c_icosahedral'                 : 'centered_icosahedral',
    'c_non'                         : 'centered_nonagonal',
    'c_non?'                        : 'nth_centered_nonagonal',
    'c_nonagonal'                   : 'centered_nonagonal',
    'c_nonagonal?'                  : 'nth_centered_nonagonal',
    'c_oct'                         : 'centered_octagonal',
    'c_oct?'                        : 'nth_centered_octagonal',
    'c_octagonal'                   : 'centered_octagonal',
    'c_octagonal?'                  : 'nth_centered_octagonal',
    'c_octahedral'                  : 'centered_octahedral',
    'c_pent'                        : 'centered_pentagonal',
    'c_pent?'                       : 'nth_centered_pentagonal',
    'c_pentagonal'                  : 'centered_pentagonal',
    'c_pentagonal?'                 : 'nth_centered_pentagonal',
    'c_poly'                        : 'centered_polygonal',
    'c_poly?'                       : 'nth_centered_polygonal',
    'c_polygonal'                   : 'centered_polygonal',
    'c_polygonal?'                  : 'nth_centered_polygonal',
    'c_sqr'                         : 'centered_square',
    'c_sqr?'                        : 'nth_centered_square',
    'c_square'                      : 'centered_square',
    'c_square?'                     : 'nth_centered_square',
    'c_tetra'                       : 'centered_tetrahedral',
    'c_tetrahedral'                 : 'centered_tetrahedral',
    'c_tri'                         : 'centered_triangular',
    'c_tri?'                        : 'nth_centered_triangular',
    'c_triangular'                  : 'centered_triangular',
    'cal'                           : 'calendar',
    'carol'                         : 'nth_carol',
    'catalan'                       : 'catalan_constant',
    'cbrt'                          : 'cube_root',
    'cbsrt'                         : 'cube_super_root',
    'cc'                            : 'centimeter^3',
    'ccube'                         : 'centered_cube',
    'ccubic'                        : 'centered_cube',
    'cdec'                          : 'centered_decagonal',
    'cdec?'                         : 'nth_centered_decagonal',
    'cdecagonal'                    : 'centered_decagonal',
    'cdecagonal?'                   : 'nth_centered_decagonal',
    'ceil'                          : 'ceiling',
    'centered_cubic'                : 'centered_cube',
    'centered_decagonal?'           : 'nth_centered_decagonal',
    'centered_heptagonal?'          : 'nth_centered_heptagonal',
    'centered_hexagonal?'           : 'nth_centered_hexagonal',
    'centered_nonagonal?'           : 'nth_centered_nonagonal',
    'centered_octagonal?'           : 'nth_centered_octagonal',
    'centered_pentagonal?'          : 'nth_centered_pentagonal',
    'centered_polygonal?'           : 'nth_centered_polygonal',
    'centered_square?'              : 'nth_centered_square',
    'centered_triangular?'          : 'nth_centered_triangular',
    'centeredcube'                  : 'centered_cube',
    'centeredcubic'                 : 'centered_cube',
    'cf'                            : 'continued_fraction',
    'champ'                         : 'champernowne_constant',
    'champernowne'                  : 'champernowne_constant',
    'chept'                         : 'centered_heptagonal',
    'chept?'                        : 'nth_centered_heptagonal',
    'cheptagonal'                   : 'centered_heptagonal',
    'cheptagonal?'                  : 'nth_centered_heptagonal',
    'chex'                          : 'centered_hexagonal',
    'chex?'                         : 'nth_centered_hexagonal',
    'chexagonal'                    : 'centered_hexagonal',
    'chexagonal?'                   : 'nth_centered_hexagonal',
    'civil_dawn'                    : 'dawn',
    'civil_dusk'                    : 'dusk',
    'click'                         : 'kilometer',
    'clicks'                        : 'kilometer',
    'cnon'                          : 'centered_nonagonal',
    'cnon?'                         : 'nth_centered_nonagonal',
    'cnonagonal'                    : 'centered_nonagonal',
    'cnonagonal?'                   : 'nth_centered_nonagonal',
    'coct'                          : 'centered_octagonal',
    'coct?'                         : 'nth_centered_octagonal',
    'coctagonal'                    : 'centered_octagonal',
    'coctagonal?'                   : 'nth_centered_octagonal',
    'comb_dig'                      : 'combine_digits',
    'combinations_of_list'          : 'get_combinations',
    'combine_dig'                   : 'combine_digits',
    'composition'                   : 'compositions',
    'conj'                          : 'conjugate',
    'contraharmonic_mean'           : 'antiharmonic_mean',
    'copeland'                      : 'copeland_erdos_constant',
    'copeland_erdos'                : 'copeland_erdos_constant',
    'cosecant'                      : 'csc',
    'cosine'                        : 'cos',
    'cotangent'                     : 'cot',
    'count_div'                     : 'count_divisors',
    'countbits'                     : 'count_bits',
    'countdiv'                      : 'count_divisors',
    'cousin'                        : 'cousin_prime',
    'cousin?'                       : 'cousin_prime',
    'cousin_'                       : 'cousin_primes',
    'cousin_prime_'                 : 'cousin_primes',
    'cousinprime'                   : 'cousin_prime',
    'cousinprime_'                  : 'cousin_primes',
    'cpent'                         : 'centered_pentagonal',
    'cpent?'                        : 'nth_centered_pentagonal',
    'cpentagonal'                   : 'centered_pentagonal',
    'cpentagonal?'                  : 'nth_centered_pentagonal',
    'cpoly'                         : 'centered_polygonal',
    'cpoly?'                        : 'nth_centered_polygonal',
    'cpolygonal'                    : 'centered_polygonal',
    'cpolygonal?'                   : 'nth_centered_polygonal',
    'csqr'                          : 'centered_square',
    'csqr?'                         : 'nth_centered_square',
    'csquare'                       : 'centered_square',
    'csquare?'                      : 'nth_centered_square',
    'ctri'                          : 'centered_triangular',
    'ctri?'                         : 'nth_centered_triangular',
    'ctriangular'                   : 'centered_triangular',
    'ctriangular?'                  : 'nth_centered_triangular',
    'cubed'                         : 'cube',
    'cuberoot'                      : 'cube_root',
    'daily'                         : '1/day',
    'day_of_week'                   : 'weekday',
    'daytime'                       : 'day_time',
    'debruijn'                      : 'debruijn_sequence',
    'deca'                          : 'decagonal',
    'deca?'                         : 'nth_decagonal',
    'decagonal?'                    : 'nth_decagonal',
    'delannoy'                      : 'nth_delannoy',
    'denomination_combinations'     : 'count_frobenius',
    'derangements'                  : 'subfactorial',
    'dice'                          : 'roll_dice',
    'dice_'                         : 'roll_dice_',
    'diff'                          : 'diffs',
    'diffs2'                        : 'cumulative_diffs',
    'dimension'                     : 'dimensions',
    'divcount'                      : 'count_divisors',
    'divides'                       : 'is_divisible',
    'divisor_count'                 : 'count_divisors',
    'double_bal'                    : 'double_balanced_prime',
    'double_bal_'                   : 'double_balanced_primes',
    'double_balanced'               : 'double_balanced_prime',
    'double_balanced_'              : 'double_balanced_primes',
    'double_balanced_prime_'        : 'double_balanced_primes',
    'doublebal'                     : 'double_balanced_prime',
    'doublebal_'                    : 'double_balanced_primes',
    'doublefac'                     : 'double_factorial',
    'dow'                           : 'weekday',
    'dup'                           : 'duplicate_term',
    'dup_dig'                       : 'duplicate_digits',
    'dup_digits'                    : 'duplicate_digits',
    'dup_number'                    : 'duplicate_number',
    'dup_op'                        : 'duplicate_operator',
    'dup_operator'                  : 'duplicate_operator',
    'dup_ops'                       : 'duplicate_operator',
    'dup_term'                      : 'duplicate_term',
    'dupop'                         : 'duplicate_operator',
    'eddington'                     : 'eddington_number',
    'egypt'                         : 'egyptian_fractions',
    'election'                      : 'election_day',
    'electron_volt'                 : 'electron-volt',
    'energy_equivalent'             : 'energy_equivalence',
    'enum_dice'                     : 'enumerate_dice',
    'enum_dice_'                    : 'enumerate_dice_',
    'equal'                         : 'is_equal',
    'equals'                        : 'is_equal',
    'ethiopian'                     : 'to_ethiopian',
    'ethiopian_name'                : 'to_ethiopian_name',
    'euler'                         : 'euler_mascheroni_constant',
    'euler_constant'                : 'euler_mascheroni_constant',
    'euler_mascheroni'              : 'euler_mascheroni_constant',
    'eulerbrick'                    : 'euler_brick',
    'eulerphi'                      : 'euler_phi',
    'eulers_number'                 : 'e',
    'eval1'                         : 'eval',
    'eval_poly'                     : 'eval_polynomial',
    'every_minute'                  : '1/minute',
    'exp_range'                     : 'exponential_range',
    'exprange'                      : 'exponential_range',
    'f!'                            : 'fibonorial',
    'fac'                           : 'factorial',
    'fac2'                          : 'double_factorial',
    'factors'                       : 'factor',
    'fall'                          : 'autumnal_equinox',
    'fermi'                         : 'femtometer',
    'fib'                           : 'fibonacci',
    'filter_list'                   : 'filter_lists',
    'find_poly'                     : 'find_polynomial',
    'find_polydivisible'            : 'generate_polydivisibles',
    'fine_structure'                : 'fine_structure_constant',
    'frac'                          : 'fraction',
    'free_space_impedance'          : 'vacuum_impedance',
    'fresnel'                       : 'terahertz',
    'fresnels'                      : 'terahertz',
    'frob'                          : 'frobenius',
    'from_indian'                   : 'from_indian_civil',
    'from_unix'                     : 'from_unix_time',
    'fromunix'                      : 'from_unix_time',
    'fromunixtime'                  : 'from_unix_time',
    'fusc'                          : 'nth_stern',
    'gamma_flux'                    : 'nanotesla',
    'gammaflux'                     : 'nanotesla',
    'gemmho'                        : 'microsiemens',
    'gen_polydiv'                   : 'generate_polydivisibles',
    'gen_polydivisibles'            : 'generate_polydivisibles',
    'geo_distance'                  : 'grographic_distance',
    'geo_mean'                      : 'geometric_mean',
    'geo_range'                     : 'geometric_range',
    'geo_recur'                     : 'geometric_recurrence',
    'geom_range'                    : 'geometric_range',
    'geomean'                       : 'geometric_mean',
    'geometric_recur'               : 'geometric_recurrence',
    'geomrange'                     : 'geometric_range',
    'georange'                      : 'geometric_range',
    'get'                           : 'get_variable',
    'get_combinations_with_repeat'  : 'get_repeat_combinations',
    'get_left_trunc'                : 'get_left_truncations',
    'get_permutations_with_repeat'  : 'get_repeat_permutations',
    'get_right_trunc'               : 'get_right_truncations',
    'gigohm'                        : 'gigaohm',
    'glaisher'                      : 'glaisher_constant',
    'golden'                        : 'phi',
    'golden_ratio'                  : 'phi',
    'greater'                       : 'is_greater',
    'greater_than'                  : 'is_greater',
    'group'                         : 'group_elements',
    'harm'                          : 'nth_harmonic_number',
    'harmonic'                      : 'nth_harmonic_number',
    'hebrew'                        : 'to_hebrew',
    'hebrew_name'                   : 'to_hebrew_name',
    'hept'                          : 'heptagonal',
    'hept?'                         : 'nth_heptagonal',
    'hept_hex'                      : 'heptagonal_hexagonal',
    'hept_pent'                     : 'heptagonal_pentagonal',
    'hept_sqr'                      : 'heptagonal_square',
    'hept_square'                   : 'heptagonal_square',
    'hept_tri'                      : 'heptagonal_triangular',
    'heptagonal?'                   : 'nth_heptagonal',
    'hepthex'                       : 'heptagonal_hexagonal',
    'heptpent'                      : 'heptagonal_pentagonal',
    'heptsqr'                       : 'heptagonal_square',
    'heptsquare'                    : 'heptagonal_square',
    'hepttri'                       : 'heptagonal_triangular',
    'hex'                           : 'hexagonal',
    'hex?'                          : 'nth_hexagonal',
    'hex_pent'                      : 'hexagonal_pentagonal',
    'hex_sqr'                       : 'hexagonal_square',
    'hex_square'                    : 'hexagonal_square',
    'hex_tri'                       : 'hexagonal',
    'hexagonal?'                    : 'nth_hexagonal',
    'hexpent'                       : 'hexagonal_pentagonal',
    'hexsqr'                        : 'hexagonal_square',
    'hexsquare'                     : 'hexagonal_square',
    'hextri'                        : 'hexagonal',
    'hipparchus'                    : 'nth_schroeder_hipparchus',
    'horizon'                       : 'horizon_distance',
    'hourly'                        : '1/hour',
    'hyper4'                        : 'tetrate',
    'hyper4_2'                      : 'tetrate_right',
    'hyper4_right'                  : 'tetrate_right',
    'hyperfac'                      : 'hyperfactorial',
    'hyperop'                       : 'hyperoperator',
    'hyperop_right'                 : 'hyperoperator_right',
    'hypot'                         : 'hypotenuse',
    'im'                            : 'imaginary',
    'index_of_max'                  : 'max_index',
    'index_of_min'                  : 'min_index',
    'indian'                        : 'to_indian_civil',
    'indian_civil'                  : 'to_indian_civil',
    'indian_civil_name'             : 'to_indian_civil_name',
    'indian_name'                   : 'to_indian_civil_name',
    'inf'                           : 'infinity',
    'int'                           : 'long',
    'int128'                        : 'quadlong',
    'int16'                         : 'short',
    'int32'                         : 'long',
    'int64'                         : 'longlong',
    'int8'                          : 'char',
    'intersect'                     : 'intersection',
    'invert'                        : 'invert_units',
    'is_armstrong'                  : 'is_narcissistic',
    'is_bimorphic'                  : 'is_automorphic',
    'is_div'                        : 'is_divisible',
    'is_int'                        : 'is_integer',
    'is_k_armstrong'                : 'is_k_narcissistic',
    'is_more'                       : 'is_greater',
    'is_morphic'                    : 'is_k_morphic',
    'is_nonequal'                   : 'is_not_equal',
    'is_not_more'                   : 'is_not_greater',
    'is_ore_harmonic'               : 'is_harmonic_divisor_number',
    'is_palindrome'                 : 'is_digital_palindrome',
    'is_plus_perfect'               : 'is_narcissistic',
    'is_spsp'                       : 'is_strong_pseudoprime',
    'is_sqr'                        : 'is_square',
    'is_unequal'                    : 'is_not_equal',
    'isdiv'                         : 'is_divisible',
    'islamic'                       : 'to_islamic',
    'islamic_name'                  : 'to_islamic_name',
    'iso_date'                      : 'to_iso',
    'isoday'                        : 'iso_day',
    'isolated'                      : 'isolated_prime',
    'isprime'                       : 'is_prime',
    'issqr'                         : 'is_square',
    'issquare'                      : 'is_square',
    'j'                             : 'i',
    'jacobsthal'                    : 'nth_jacobsthal',
    'jalali'                        : 'to_persian',
    'jalali_name'                   : 'to_persian_name',
    'julian_day'                    : 'to_julian_day',
    'julianday'                     : 'to_julian_day',
    'khinchin'                      : 'khinchin_constant',
    'klick'                         : 'kilometer',
    'klicks'                        : 'kilometer',
    'klik'                          : 'kilometer',
    'kliks'                         : 'kilometer',
    'kph'                           : 'kilometer/hour',
    'ksphere_area'                  : 'k_sphere_area',
    'ksphere_radius'                : 'k_sphere_radius',
    'ksphere_volume'                : 'k_sphere_volume',
    'kspherearea'                   : 'k_sphere_area',
    'ksphereradius'                 : 'k_sphere_radius',
    'kspherevolume'                 : 'k_sphere_volume',
    'kynea'                         : 'nth_kynea',
    'lP'                            : 'planck_length',
    'lah'                           : 'lah_number',
    'largest'                       : 'maximum',
    'latlong'                       : 'lat_long',
    'latlong_to_nac'                : 'lat_long_to_nac',
    'len'                           : 'count',
    'length'                        : 'count',
    'leonardo'                      : 'nth_leonardo',
    'less'                          : 'is_less',
    'less_than'                     : 'is_less',
    'leyland'                       : 'leyland_number',
    'lgamma'                        : 'log_gamma',
    'lilian_day'                    : 'to_lilian_day',
    'linear'                        : 'linear_recurrence',
    'linear_recur'                  : 'linear_recurrence',
    'linearrecur'                   : 'linear_recurrence',
    'little_schroeder'              : 'nth_schroeder_hipparchus',
    'll_nac'                        : 'latlong_to_nac',
    'll_to_nac'                     : 'latlong_to_nac',
    'ln'                            : 'log',
    'loggamma'                      : 'log_gamma',
    'mP'                            : 'planck_mass',
    'make_cf'                       : 'make_continued_fraction',
    'make_date'                     : 'make_datetime',
    'make_iso'                      : 'make_iso_time',
    'make_julian'                   : 'make_julian_time',
    'make_pyth3'                    : 'make_pyth_3',
    'make_pyth4'                    : 'make_pyth_4',
    'make_time'                     : 'make_datetime',
    'makecf'                        : 'make_continued_fraction',
    'makeiso'                       : 'make_iso_time',
    'makeisotime'                   : 'make_iso_time',
    'makejulian'                    : 'make_julian_time',
    'makejuliantime'                : 'make_julian_time',
    'makepyth'                      : 'make_pyth_3',
    'makepyth3'                     : 'make_pyth_3',
    'makepyth4'                     : 'make_pyth_4',
    'maketime'                      : 'make_datetime',
    'math'                          : 'arithmetic',
    'max'                           : 'maximum',
    'mayan'                         : 'to_mayan',
    'mcg'                           : 'microgram',
    'megalerg'                      : 'megaerg',
    'megaohm'                       : 'megohm',
    'megmho'                        : 'megasiemens',
    'melting_point'                 : 'element_melting_point',
    'menage'                        : 'nth_menage',
    'mersenne'                      : 'nth_mersenne_prime',
    'mersenne_exponent'             : 'nth_mersenne_exponent',
    'mersenne_power'                : 'nth_mersenne_exponent',
    'mersenne_prime'                : 'nth_mersenne_prime',
    'merten'                        : 'nth_merten',
    'mertens'                       : 'merten',
    'mertens_constant'              : 'merten_constant',
    'metre'                         : 'meter',
    'metres'                        : 'meter',
    'mills'                         : 'mills_constant',
    'min'                           : 'minimum',
    'mlk_day'                       : 'martin_luther_king_day',
    'mobius'                        : 'nth_mobius',
    'mod'                           : 'modulo',
    'modulus'                       : 'abs',
    'moebius'                       : 'nth_mobius',
    'molar_weight'                  : 'molar_mass',
    'mondo_point'                   : 'millimeter',
    'monthly'                       : '1/month',
    'more'                          : 'is_greater',
    'more_than'                     : 'is_greater',
    'motzkin'                       : 'nth_motzkin',
    'mph'                           : 'mile/hour',
    'mu'                            : 'mobius',
    'mul_poly'                      : 'multiply_polynomials',
    'mult'                          : 'multiply',
    'mult_dig'                      : 'multiply_digits',
    'mult_digits'                   : 'multiply_digits',
    'mult_poly'                     : 'multiply_polynomials',
    'multifac'                      : 'multifactorial',
    'multiplex'                     : 'permute_lists',
    'n_fibonacci'                   : 'k_fibonacci',
    'n_persistence'                 : 'k_persistence',
    'n_sphere_area'                 : 'k_sphere_area',
    'n_sphere_radius'               : 'k_sphere_radius',
    'n_sphere_volume'               : 'k_sphere_volume',
    'napiers_constant'              : 'e',
    'narayana'                      : 'narayan_number',
    'neg'                           : 'negative',
    'negate'                        : 'not',
    'next_first_quarter'            : 'next_first_quarter_moon',
    'next_full'                     : 'next_full_moon',
    'next_last_quarter'             : 'next_last_quarter_moon',
    'next_new'                      : 'next_new_moon',
    'next_prime_index'              : 'nth_prime',
    'nighttime'                     : 'night_time',
    'ninf'                          : 'negative_infinity',
    'nint'                          : 'nearest_int',
    'nlimit'                        : 'limit',
    'non'                           : 'nonagonal',
    'non?'                          : 'nth_nonagonal',
    'nona_hept'                     : 'nonagonal_heptagonal',
    'nona_hex'                      : 'nonagonal_hexagonal',
    'nona_oct'                      : 'nonagonal_octagonal',
    'nona_pent'                     : 'nonagonal_pentagonal',
    'nona_sqr'                      : 'nonagonal_square',
    'nona_square'                   : 'nonagonal_square',
    'nona_tri'                      : 'nonagonal_triangular',
    'nonagonal?'                    : 'nth_nonagonal',
    'nonahept'                      : 'nonagonal_heptagonal',
    'nonahex'                       : 'nonagonal_hexagonal',
    'nonaoct'                       : 'nonagonal_octagonal',
    'nonapent'                      : 'nonagonal_pentagonal',
    'nonasqr'                       : 'nonagonal_square',
    'nonasquare'                    : 'nonagonal_square',
    'nonatri'                       : 'nonagonal_triangular',
    'nonzeroes'                     : 'nonzero',
    'not_equals'                    : 'is_not_equal',
    'not_greater'                   : 'is_not_greater',
    'not_greater_than'              : 'is_not_greater',
    'not_less_than'                 : 'is_not_less',
    'not_more_than'                 : 'is_not_greater',
    'nsphere_area'                  : 'k_sphere_area',
    'nsphere_radius'                : 'k_sphere_radius',
    'nsphere_volume'                : 'k_sphere_volume',
    'nspherearea'                   : 'k_sphere_area',
    'nsphereradius'                 : 'k_sphere_radius',
    'nspherevolume'                 : 'k_sphere_volume',
    'nth_fibonacci'                 : 'fibonacci',
    'nth_harmonic'                  : 'nth_harmonic_number',
    'nth_k_fibonacci'               : 'k_fibonacci',
    'nth_little_schroeder'          : 'nth_schroeder_hipparchus',
    'nth_mersenne'                  : 'nth_mersenne_prime',
    'nth_perfect'                   : 'nth_perfect_number',
    'nth_prime?'                    : 'nth_prime',
    'nth_quad?'                     : 'nth_quadruplet_prime',
    'nth_quint?'                    : 'nth_quintuplet_prime',
    'nth_sqr'                       : 'nth_square',
    'nth_super_catalan'             : 'nth_schroeder_hipparchus',
    'nth_tri'                       : 'nth_triangular',
    'nthday'                        : 'nth_weekday',
    'nthdayofyear'                  : 'nth_weekday_of_year',
    'nthprime'                      : 'prime',
    'nthprime?'                     : 'nth_prime',
    'nthquad?'                      : 'nth_quadruplet_prime',
    'nthquint?'                     : 'nth_quintuplet_prime',
    'nthweekday'                    : 'nth_weekday',
    'nthweekdayofyear'              : 'nth_weekday_of_year',
    'oct_hept'                      : 'octagonal_heptagonal',
    'oct_hex'                       : 'octagonal_hexagonal',
    'oct_pent'                      : 'octagonal_pentagonal',
    'oct_sqr'                       : 'octagonal_square',
    'oct_square'                    : 'octagonal_square',
    'oct_tri'                       : 'octagonal_triangular',
    'octa'                          : 'octagonal',
    'octa?'                         : 'nth_octagonal',
    'octagonal?'                    : 'nth_octagonal',
    'octhept'                       : 'octagonal_heptagonal',
    'octhex'                        : 'octagonal_hexagonal',
    'octpent'                       : 'octagonal_pentagonal',
    'octsqr'                        : 'octagonal_square',
    'octsquare'                     : 'octagonal_square',
    'octtri'                        : 'octagonal_triangular',
    'octy_prime_'                   : 'octy_primes',
    'oeiscomment'                   : 'oeis_comment',
    'oeisex'                        : 'oeis_ex',
    'oeisname'                      : 'oeis_name',
    'omega'                         : 'omega_constant',
    'ordinal'                       : 'ordinal_name',
    'p!'                            : 'primorial',
    'padovan'                       : 'nth_padovan',
    'partition'                     : 'partitions',
    'pascal_tri'                    : 'pascal_triangle',
    'pascaltri'                     : 'pascal_triangle',
    'pell'                          : 'nth_pell',
    'pent'                          : 'pentagonal',
    'pent?'                         : 'nth_pentagonal',
    'pent_sqr'                      : 'pentagonal_square',
    'pent_square'                   : 'pentagonal_square',
    'pent_tri'                      : 'pentagonal_triangular',
    'pentagonal?'                   : 'nth_pentagonal',
    'pentsqr'                       : 'pentagonal_square',
    'pentsquare'                    : 'pentagonal_square',
    'penttri'                       : 'pentagonal_triangular',
    'perfect'                       : 'nth_perfect_number',
    'perm'                          : 'permutations',
    'permutations_of_list'          : 'get_permutations',
    'persian'                       : 'to_persian',
    'persian_name'                  : 'to_persian_name',
    'plastic'                       : 'plastic_constant',
    'plotc'                         : 'plot_complex',
    'poiseuille'                    : 'pascal*second',
    'poiseuilles'                   : 'pascal*second',
    'poly'                          : 'polygonal',
    'poly*'                         : 'multiply_polynomials',
    'poly**'                        : 'polynomial_power',
    'poly+'                         : 'add_polynomials',
    'poly?'                         : 'nth_polygonal',
    'poly_10_3'                     : 'decagonal_triangular',
    'poly_10_5'                     : 'decagonal_pentagonal',
    'poly_10_6'                     : 'decagonal_hexagonal',
    'poly_10_7'                     : 'decagonal_heptagonal',
    'poly_10_8'                     : 'decagonal_octagonal',
    'poly_10_9'                     : 'decagonal_nonagonal',
    'poly_10_c4'                    : 'decagonal_centered_square',
    'poly_4_3'                      : 'square_triangular',
    'poly_5_3'                      : 'pentagonal_triangular',
    'poly_5_4'                      : 'pentagonal_square',
    'poly_6_3'                      : 'hexagonal',
    'poly_6_4'                      : 'hexagonal_square',
    'poly_6_5'                      : 'hexagonal_pentagonal',
    'poly_7_3'                      : 'heptagonal_triangular',
    'poly_7_4'                      : 'heptagonal_square',
    'poly_7_5'                      : 'heptagonal_pentagonal',
    'poly_7_6'                      : 'heptagonal_hexagonal',
    'poly_8_3'                      : 'octagonal_triangular',
    'poly_8_4'                      : 'octagonal_square',
    'poly_8_5'                      : 'octagonal_pentagonal',
    'poly_8_6'                      : 'octagonal_hexagonal',
    'poly_8_7'                      : 'octagonal_heptagonal',
    'poly_9_3'                      : 'nonagonal_triangular',
    'poly_9_4'                      : 'nonagonal_square',
    'poly_9_5'                      : 'nonagonal_pentagonal',
    'poly_9_6'                      : 'nonagonal_hexagonal',
    'poly_9_7'                      : 'nonagonal_heptagonal',
    'poly_9_8'                      : 'nonagonal_octagonal',
    'poly_mult'                     : 'multiply_polynomials',
    'poly_power'                    : 'polynomial_power',
    'poly_prod'                     : 'polynomial_product',
    'poly_product'                  : 'polynomial_product',
    'poly_sum'                      : 'polynomial_sum',
    'polyadd'                       : 'add_polynomials',
    'polyarea'                      : 'polygon_area',
    'polyeval'                      : 'eval_polynomial',
    'polygonal?'                    : 'nth_polygonal',
    'polymul'                       : 'multiply_polynomials',
    'polymult'                      : 'multiply_polynomials',
    'polypower'                     : 'polynomial_power',
    'polyprod'                      : 'polynomial_product',
    'polyproduct'                   : 'polynomial_product',
    'polysum'                       : 'polynomial_sum',
    'polyval'                       : 'eval_polynomial',
    'prev'                          : 'previous',
    'prev_first_quarter'            : 'previous_first_quarter_moon',
    'prev_full_moon'                : 'previous_full_moon',
    'prev_last_quarter'             : 'previous_last_quarter_moon',
    'prev_new_moon'                 : 'previous_new_moon',
    'prev_prime'                    : 'previous_prime',
    'prev_primes'                   : 'previous_primes',
    'previous_first_quarter'        : 'previous_first_quarter_moon',
    'previous_full'                 : 'previous_full_moon',
    'previous_last_quarter'         : 'previous_last_quarter_moon',
    'previous_new'                  : 'previous_new_moon',
    'prevost'                       : 'prevost_constant',
    'prime?'                        : 'next_prime',
    'primepi'                       : 'prime_pi',
    'primitive'                     : 'primitive_units',
    'prod'                          : 'product',
    'puff'                          : 'picofarad',
    'pyr'                           : 'pyramidal',
    'pyramid'                       : 'pyramidal',
    'qP'                            : 'planck_charge',
    'quad?'                         : 'nth_quadruplet_prime',
    'quad_prime'                    : 'quadruplet_prime',
    'quad_prime?'                   : 'nth_quadruplet_prime',
    'quad_prime_'                   : 'quadruplet_primes',
    'quad_primes_'                  : 'quadruplet_primes',
    'quadprime'                     : 'quadruplet_prime',
    'quadprime?'                    : 'nth_quadruplet_prime',
    'quadprime_'                    : 'quadruplet_primes',
    'quadruple_bal'                 : 'quadruple_balanced',
    'quadruple_bal_'                : 'quadruple_balanceds',
    'quadruple_balanced'            : 'quadruple_balanced_prime',
    'quadruple_balanced_'           : 'quadruple_balanced_primes',
    'quadruple_balanced_prime_'     : 'quadruple_balanced_primes',
    'quadruplet_prime_'             : 'quadruplet_primes',
    'quadruplet_primes_'            : 'quadruplet_primes',
    'quint'                         : 'quintuplet_prime',
    'quint?'                        : 'nth_quintuplet_prime',
    'quint_'                        : 'quintuplet_primes',
    'quint_prime'                   : 'quintuplet_prime',
    'quint_prime?'                  : 'nth_quintuplet_prime',
    'quint_prime_'                  : 'quintuplet_primes',
    'quint_primes_'                 : 'quintuplet_primes',
    'quintprime'                    : 'quintuplet_prime',
    'quintprime?'                   : 'nth_quintuplet_prime',
    'quintprime_'                   : 'quintuplet_primes',
    'quintuplet_prime?'             : 'next_quintuplet_prime',
    'quintuplet_prime_'             : 'quintuplet_primes',
    'quintuplet_primes_'            : 'quintuplet_primes',
    'rand'                          : 'random',
    'rand_int'                      : 'random_integer',
    'rand_int_'                     : 'random_integers',
    'rand_integers'                 : 'random_integers',
    'rand_ints'                     : 'random_integers',
    'randint'                       : 'random_integer',
    'randint_'                      : 'random_integers',
    'randints'                      : 'random_integers',
    'random_int'                    : 'random_integer',
    'random_int_'                   : 'random_integers',
    'random_integer_'               : 'random_integers',
    'random_ints'                   : 'random_integers',
    'range2'                        : 'interval_range',
    'range3'                        : 'sized_range',
    'ratios2'                       : 'cumulative_ratios',
    're'                            : 'real',
    'rev_add'                       : 'reversal_addition',
    'rev_dig'                       : 'reverse_digits',
    'rev_digits'                    : 'reverse_digits',
    'rhombdodec'                    : 'rhombic_dodecahedral',
    'rms'                           : 'root_mean_square',
    'robbins'                       : 'robbins_constant',
    'root2'                         : 'square_root',
    'root3'                         : 'cube_root',
    'roots_of_unity'                : 'unit_roots',
    'rsort'                         : 'sort_descending',
    'safe'                          : 'safe_prime',
    'safeprime'                     : 'safe_prime',
    'schroeder'                     : 'nth_schroeder',
    'schroeder_hipparchus'          : 'nth_schroeder_hipparchus',
    'schwarzchild'                  : 'black_hole_radius',
    'schwarzchild_radius'           : 'black_hole_radius',
    'secant'                        : 'sec',
    'set'                           : 'set_variable',
    'sext'                          : 'sextuplet_prime',
    'sext_'                         : 'sextuplet_primes',
    'sext_prime'                    : 'sextuplet_prime',
    'sext_prime_'                   : 'sextuplet_primes',
    'sextprime'                     : 'sextuplet_prime',
    'sextprime_'                    : 'sextuplet_primes',
    'sextuplet_prime_'              : 'sextuplet_primes',
    'sexy'                          : 'sexy_prime',
    'sexy3'                         : 'sexy_triplet',
    'sexy3_'                        : 'sexy_triplets',
    'sexy4'                         : 'sexy_quadruplet',
    'sexy4_'                        : 'sexy_quadruplets',
    'sexy_'                         : 'sexy_prime_',
    'sexy_prime_'                   : 'sexy_primes',
    'sexy_quad'                     : 'sexy_quadruplet',
    'sexy_quad_'                    : 'sexy_quadruplets',
    'sexy_quadruplet_'              : 'sexy_quadruplets',
    'sexy_triplet_'                 : 'sexy_triplets',
    'sexyprime'                     : 'sexy_prime',
    'sexyprime_'                    : 'sexy_primes',
    'sexyquad'                      : 'sexy_quadruplet',
    'sexyquad_'                     : 'sexy_quadruplets',
    'sexytriplet'                   : 'sexy_triplet',
    'sexytriplet_'                  : 'sexy_triplets',
    'shiftleft'                     : 'shift_left',
    'shiftright'                    : 'shift_right',
    'show_n_persistence'            : 'show_k_persistence',
    'sigma_n'                       : 'sigma_k',
    'sigma_sb'                      : 'stefan_boltzmann_constant',
    'silver'                        : 'silver_ratio',
    'sine'                          : 'sin',
    'sleft'                         : 'shift_left',
    'slice2'                        : 'sublist',
    'smallest'                      : 'minimum',
    'solve2'                        : 'solve_quadratic',
    'solve3'                        : 'solve_cubic',
    'solve4'                        : 'solve_quartic',
    'solve_2'                       : 'solve_quadratic',
    'solve_3'                       : 'solve_cubic',
    'solve_4'                       : 'solve_quartic',
    'sophie'                        : 'sophie_prime',
    'sophieprime'                   : 'sophie_prime',
    'sort_desc'                     : 'sort_descending',
    'sortdesc'                      : 'sort_descending',
    'spherearea'                    : 'sphere_area',
    'sphereradius'                  : 'sphere_radius',
    'spherevolume'                  : 'sphere_volume',
    'split'                         : 'unpack',
    'spring'                        : 'vernal_equinox',
    'sqr'                           : 'square',
    'sqr_tri'                       : 'square_triangular',
    'sqrt'                          : 'square_root',
    'sqrtri'                        : 'square_triangular',
    'sqsrt'                         : 'square_super_root',
    'square?'                       : 'nth_square',
    'square_tri'                    : 'square_triangular',
    'squared'                       : 'square',
    'squareroot'                    : 'square_root',
    'squaretri'                     : 'square_triangular',
    'sright'                        : 'shift_right',
    'star_number'                   : 'star',
    'stefan_boltzmann'              : 'stefan_boltzmann_constant',
    'stefan_constant'               : 'stefan_boltzmann_constant',
    'stelloct'                      : 'stella_octangula',
    'stern'                         : 'nth_stern',
    'stirling1'                     : 'stirling1_number',
    'stirling2'                     : 'stirling2_number',
    'subfac'                        : 'subfactorial',
    'sum_dig'                       : 'sum_digits',
    'summer'                        : 'summer_solstice',
    'sun_transit'                   : 'solar_noon',
    'super_catalan'                 : 'nth_schroeder_hipparchus',
    'superfac'                      : 'superfactorial',
    'superprime'                    : 'super_prime',
    'syl'                           : 'nth_sylvester',
    'sylvester'                     : 'nth_sylvester',
    'tP'                            : 'planck_time',
    'tangent'                       : 'tan',
    'thabit'                        : 'nth_thabit',
    'thue_morse'                    : 'nth_thue_morse',
    'times'                         : 'multiply',
    'to_jalali'                     : 'to_persian',
    'to_lilian'                     : 'to_lilian_day',
    'to_ordinal'                    : 'to_ordinal_date',
    'to_unix'                       : 'to_unix_time',
    'totient'                       : 'euler_phi',
    'tounix'                        : 'to_unix_time',
    'tounixtime'                    : 'to_unix_time',
    'tower'                         : 'power_tower',
    'tower2'                        : 'power_tower2',
    'tri'                           : 'triangular',
    'tri?'                          : 'nth_triangular',
    'tri_area'                      : 'triangle_area',
    'tri_sqr'                       : 'square_triangular',
    'trianglearea'                  : 'triangle_area',
    'triangular?'                   : 'nth_triangular',
    'triarea'                       : 'triangle_area',
    'trib'                          : 'tribonacci',
    'triple_bal'                    : 'triple_balanced',
    'triple_bal_'                   : 'triple_balanceds',
    'triple_balanced'               : 'triple_balanced_prime',
    'triple_balanced_'              : 'triple_balanced_primes',
    'triple_balanced_prime_'        : 'triple_balanced_primes',
    'triplet'                       : 'triplet_prime',
    'triplet_'                      : 'triplet_primes',
    'triplet_prime_'                : 'triplet_primes',
    'triplet_primes_'               : 'triplet_primes',
    'tripletprime'                  : 'triplet_prime',
    'tripletprime_'                 : 'triplet_primes',
    'trisqr'                        : 'square_triangular',
    'trunc_oct'                     : 'truncated_octahedral',
    'trunc_tet'                     : 'truncated_tetrahedral',
    'truncoct'                      : 'truncated_octahedral',
    'trunctet'                      : 'truncated_tetrahedral',
    'twin'                          : 'twin_prime',
    'twin_'                         : 'twin_primes',
    'twin_prime_'                   : 'twin_primes',
    'twin_primes_'                  : 'twin_primes',
    'twinprime'                     : 'twin_prime',
    'twinprime_'                    : 'twin_primes',
    'uint'                          : 'ulong',
    'uint128'                       : 'uquadlong',
    'uint16'                        : 'ushort',
    'uint32'                        : 'ulong',
    'uint64'                        : 'ulonglong',
    'uint8'                         : 'uchar',
    'unitroots'                     : 'unit_roots',
    'units'                         : 'unit_types',
    'unsigned'                      : 'uinteger',
    'variable'                      : 'get_variable',
    'vernal'                        : 'vernal_equinox',
    'von_klitzing'                  : 'von_klitzing_constant',
    'weekly'                        : '1/week',
    'winter'                        : 'winter_solstice',
    'year_cal'                      : 'year_calendar',
    'yearcal'                       : 'year_calendar',
    'yearcalendar'                  : 'year_calendar',
    'yearly'                        : '1/year',
    'z0'                            : 'vacuum_impedance',
    'z_0'                           : 'vacuum_impedance',
    'z_nought'                      : 'vacuum_impedance',
    'z_sub_0'                       : 'vacuum_impedance',
    'zero_mode'                     : 'leading_zero_mode',
    'zeroes'                        : 'zero',
    '{'                             : '(',
    '|'                             : 'is_divisible',
    '}'                             : ')',
    '~'                             : 'bitwise_not',

}




#!/usr/bin/env python

#******************************************************************************
#
#  rpnExpression.py
#
#  rpnChilada expression class and related utilities
#  copyright (c) 2020, Rick Gutleber (rickg@his.com)
#
#  License: GNU GPL 3.0 (see <http://www.gnu.org/licenses/gpl.html> for more
#  information).
#
#******************************************************************************

from mpmath import mpmathify

from rpn.rpnDebug import debugPrint

import rpn.rpnGlobals as g


#******************************************************************************
#
#  class RPNExpression
#
#******************************************************************************

class RPNExpression( object ):
    '''This class defines an expression used for parsing rpnChilada.'''
    def __init__( self, operators, arguments ):
        self.operators = [ ]
        self.argumemts = [ ]

    def parse( self ):
        '''.'''
        return self




#!/usr/bin/env python

#******************************************************************************
#
#  rpnVersion.py
#
#  rpnChilada version identification
#  copyright (c) 2020, Rick Gutleber (rickg@his.com)
#
#  License: GNU GPL 3.0 (see <http://www.gnu.org/licenses/gpl.html> for more
#  information).
#
#******************************************************************************

#******************************************************************************
#
#  version variable initialization
#
#******************************************************************************

PROGRAM_NAME = 'rpnChilada'
PROGRAM_VERSION = '8.4.0'
PROGRAM_VERSION_NAME = '8.4.0'
COPYRIGHT_MESSAGE = 'copyright (c) 2020 (1988), Rick Gutleber (rickg@his.com)'

if PROGRAM_VERSION != PROGRAM_VERSION_NAME:
    PROGRAM_VERSION_STRING = ' ' + PROGRAM_VERSION + ' (' + PROGRAM_VERSION_NAME + ')'
else:
    PROGRAM_VERSION_STRING = ' ' + PROGRAM_VERSION

RPN_PROGRAM_NAME = PROGRAM_NAME + PROGRAM_VERSION_STRING

PROGRAM_DESCRIPTION = 'RPN command-line calculator'




#!/usr/bin/env python

#******************************************************************************
#
#  rpnRational.py
#
#  rpnChilada rational value class
#  copyright (c) 2020, Rick Gutleber (rickg@his.com)
#
#  License: GNU GPL 3.0 (see <http://www.gnu.org/licenses/gpl.html> for more
#  information).
#
#******************************************************************************

from fractions import Fraction


#******************************************************************************
#
#  class RPNRational
#
#  Maybe I'll get around to implementing this some day.
#
#******************************************************************************

class RPNRational( Fraction ):
    '''This class represents a rational value.  It is derived from
    fractions.Fraction, but has support for dealing with mpf, etc..'''

    def __init__( self, value ):
        self = value

    def __eq__( self, other ):
        return Fraction.__eq__( other )

    def __ne__( self, other ):
        return __ne__( self, other )




#!/usr/bin/env python

#******************************************************************************
#
#  rpnOperator.py
#
#  rpnChilada operator class definitions
#  copyright (c) 2020, Rick Gutleber (rickg@his.com)
#
#  License: GNU GPL 3.0 (see <http://www.gnu.org/licenses/gpl.html> for more
#  information).
#
#******************************************************************************

import random

from enum import Enum

from mpmath import exp, fadd, fmul, floor, im, nan, nstr

from rpn.rpnDateTime import RPNDateTime
from rpn.rpnGenerator import RPNGenerator
from rpn.rpnLocation import RPNLocation
from rpn.rpnMeasurement import RPNMeasurement
from rpn.rpnSpecial import getRandomInteger, getRandomNumber
from rpn.rpnUtils import abortArgsNeeded


import rpn.rpnGlobals as g


#******************************************************************************
#
#  RPNArgumentType class
#
#******************************************************************************

class RPNArgumentType( Enum ):
    Default = 0                 # any argument is valid
    Real = 1
    NonnegativeReal = 2         # real >= 0
    Integer = 3
    PositiveInteger = 4         # integer >= 1
    NonnegativeInteger = 5      # integer >= 0
    PrimeInteger = 649

    String = 7
    DateTime = 8
    Location = 9                # location object (operators will automatically convert a string)
    Boolean = 10                # 0 or 1
    Measurement = 11
    AstronomicalObject = 12
    List = 13                   # the argument must be a list
    Generator = 14              # Generator is a separate type now, but eventually it should be equivalent to List
    Function = 15


#******************************************************************************
#
#  generateDefaultArgument
#
#******************************************************************************

def generateDefaultArgument( ):
    return 'argument'


#******************************************************************************
#
#  generateRealArgument
#
#  for real numbers we are treating the range as logarithms
#
#******************************************************************************

def generateRealArgument( range = None, allowNegative = True ):
    if range is None:
        range = [ 0, 10 ]

    factor = 1

    if allowNegative and getRandomInteger( 2 ) == 1:
        factor = -1

    return nstr( fmul( exp( fmul( getRandomNumber( ), random.uniform( *range ) ) ), factor ) )


#******************************************************************************
#
#  generateNonnegativeRealArgument
#
#******************************************************************************

def generateNonnegativeRealArgument( range ):
    return generateRealArgument( range, allowNegative=False )


def generateIntegerArgument( range = None, allowNegative=True ):
    if range is None:
        range = [ 0, 1_000_000_000 ]

    factor = 1

    if allowNegative and getRandomInteger( 2 ) == 1:
        factor = -1

    return str( fmul( fadd( getRandomInteger( range[ 1 ] ), range[ 0 ] ), factor ) )


def generatePositiveIntegerArgument( range=None ):
    if range is None:
        range = [ 1, 1_000_000_000 ]

    return generateIntegerArgument( range, allowNegative=False )


def generateNonnegativeIntegerArgument( range=None ):
    if range is None:
        range = [ 0, 1_000_000_000 ]

    return generateIntegerArgument( range, allowNegative=False )


def generatePrimeArgument( ):
    return 'argument'


def generateStringArgument( ):
    return 'argument'


def generateDateTimeArgument( ):
    return 'argument'


def generateLocationArgument( ):
    return 'argument'


def generateBooleanArgument( ):
    return 'argument'


def generateMeasurementArgument( ):
    return 'argument'


def generateAstronomicalObjectArgument( ):
    return 'argument'


def generateListArgument( ):
    return 'argument'


def generateGeneratorArgument( ):
    return 'argument'


def generateFunctionArgument( ):
    return 'argument'


#******************************************************************************
#
#  argumentGenerators dict
#
#******************************************************************************

argumentGenerators = {
    RPNArgumentType.Default             : generateDefaultArgument,
    RPNArgumentType.Real                : generateRealArgument,
    RPNArgumentType.NonnegativeReal     : generateNonnegativeRealArgument,
    RPNArgumentType.Integer             : generateIntegerArgument,
    RPNArgumentType.PositiveInteger     : generatePositiveIntegerArgument,
    RPNArgumentType.NonnegativeInteger  : generateNonnegativeIntegerArgument,
    RPNArgumentType.PrimeInteger        : generatePrimeArgument,
    RPNArgumentType.String              : generateStringArgument,
    RPNArgumentType.DateTime            : generateDateTimeArgument,
    RPNArgumentType.Location            : generateLocationArgument,
    RPNArgumentType.Boolean             : generateBooleanArgument,
    RPNArgumentType.Measurement         : generateMeasurementArgument,
    RPNArgumentType.AstronomicalObject  : generateAstronomicalObjectArgument,
    RPNArgumentType.List                : generateListArgument,
    RPNArgumentType.Generator           : generateGeneratorArgument,
    RPNArgumentType.Function            : generateFunctionArgument,
}


#******************************************************************************
#
#  class RPNVariable
#
#******************************************************************************

class RPNVariable( ):
    '''
    This class represents a variable in rpn, and it maintains a global
    dictionary of all variables keyed by name.
    '''
    def __init__( self, name ):
        self.name = name
        self.isHistory = not name[ 0 ].isalpha( )

        if name in g.variables:
            self.value = g.variables[ name ]
        else:
            self.value = nan

    def setValue( self, value ):
        if self.isHistory:
            raise ValueError( 'cannot set the value of a history expression' )

        self.value = value
        g.variables[ self.name ] = value

    def getValue( self ):
        if self.isHistory:
            prompt = int( self.name )

            if 0 < prompt < g.promptCount:
                self.value = g.results[ prompt ]
            else:
                raise ValueError( 'result index out of range' )

        return self.value


#******************************************************************************
#
#  class RPNOperator
#
#******************************************************************************

class RPNOperator( ):
    measurementsAllowed = True
    measurementsNotAllowed = False

    '''This class represents all the data needed to define an operator.'''
    def __init__( self, function, argCount, argTypes = None, argTestRanges = None,
                  allowMeasurements = measurementsNotAllowed ):
        self.function = function
        self.argCount = argCount

        if argTypes is None:
            self.argTypes = list( )
        else:
            self.argTypes = argTypes

        if argTestRanges is None:
            self.argTestRanges = list( )
        else:
            self.argTestRanges = argTestRanges

        self.allowMeasurements = allowMeasurements

    # This method isn't used yet, but I hope to start using it soon.
    @staticmethod
    def validateArgType( term, arg, argType ):
        if isinstance( arg, ( list, RPNGenerator ) ) and \
           argType not in ( RPNArgumentType.List, RPNArgumentType.Generator ):
            return True

        if argType == RPNArgumentType.Default:
            return True
        elif argType == RPNArgumentType.Real and im( arg ):
            raise ValueError( '\'' + term + '\':  real argument expected' )
        elif argType == RPNArgumentType.NonnegativeReal and ( im( arg ) or arg < 0 ):
            raise ValueError( '\'' + term + '\':  non-negative real argument expected' )
        elif argType == RPNArgumentType.Integer and arg != floor( arg ):
            raise ValueError( '\'' + term + '\':  integer argument expected' )
        elif argType == RPNArgumentType.NonnegativeInteger and arg != floor( arg ) or arg < 0:
            raise ValueError( '\'' + term + '\':  non-negative integer argument expected' )
        elif argType == RPNArgumentType.PositiveInteger and arg != floor( arg ) or arg < 1:
            raise ValueError( '\'' + term + '\':  positive integer argument expected' )
        elif argType == RPNArgumentType.String and not isinstance( arg, str ):
            raise ValueError( '\'' + term + '\':  string argument expected' )
        elif argType == RPNArgumentType.DateTime and not isinstance( arg, RPNDateTime ):
            raise ValueError( '\'' + term + '\':  date-time argument expected' )
        elif argType == RPNArgumentType.Location and not isinstance( arg, ( RPNLocation, str ) ):
            raise ValueError( '\'' + term + '\':  location argument expected' )
        elif argType == RPNArgumentType.Boolean and arg != 0 and arg != 1:
            raise ValueError( '\'' + term + '\':  boolean argument expected (0 or 1)' )
        elif argType == RPNArgumentType.Measurement and not isinstance( arg, RPNMeasurement ):
            raise ValueError( '\'' + term + '\':  measurement argument expected' )
        elif argType == RPNArgumentType.AstronomicalObject:
            pass
        elif argType == RPNArgumentType.List and not isinstance( arg, ( list, RPNGenerator ) ):
            raise ValueError( '\'' + term + '\':  list argument expected' )
        elif argType == RPNArgumentType.Generator and not isinstance( arg, RPNGenerator ):
            raise ValueError( '\'' + term + '\':  generator argument expected' )
        elif argType == RPNArgumentType.Function and not isinstance( arg, RPNFunction ):
            raise ValueError( '\'' + term + '\':  function argument expected' )

        return False

    def evaluate( self, term, index, currentValueList ):
        # handle a regular operator
        argsNeeded = self.argCount

        # first we validate, and make sure the operator has enough arguments
        if len( currentValueList ) < argsNeeded:
            abortArgsNeeded( term, index, argsNeeded )
            return False

        if argsNeeded == 0:
            result = self.function( )
        else:
            argList = list( )

            if g.operatorList:
                g.operatorsInList += 1

            # build argument list
            for i in range( 0, argsNeeded ):
                if g.operatorList:
                    # we need to tee a generator so it can run more than once
                    if isinstance( currentValueList[ g.lastOperand - 1 ], RPNGenerator ):
                        arg = currentValueList[ g.lastOperand - i ].clone( )
                    else:
                        arg = currentValueList[ g.lastOperand - i ]

                    if argsNeeded > g.operandsToRemove:
                        g.operandsToRemove = argsNeeded
                else:
                    arg = checkForVariable( currentValueList.pop( ) )

                    if term != 'set_variable' and isinstance( arg, RPNVariable ):
                        raise ValueError( 'set_variable called with a nonvariable argument' )
                        #arg = arg.getValue( )

                argList.append( arg if isinstance( arg, ( list, RPNGenerator ) ) else [ arg ] )

            # argument validation
            #self.validateArgTypes( term, *reverse( argList ), self.argTypes[ i ] )

            #print( 'argList', *reversed( argList ) )
            #print( 'self.function', self.function )

            result = callers[ argsNeeded ]( self.function, *reversed( argList ) )
            #result = list( map( self.function, *reversed( argList ) ) )

        # process results
        newResult = list( )

        if isinstance( result, RPNGenerator ):
            newResult.append( result )
        else:
            if not isinstance( result, list ):
                result = [ result ]

            for item in result:
                if isinstance( item, RPNMeasurement ) and item.units == { }:
                    newResult.append( item.value )
                else:
                    newResult.append( item )

        if len( newResult ) == 1:
            newResult = newResult[ 0 ]

        if term not in sideEffectOperators:
            currentValueList.append( newResult )

        return True

    def generateCall( self, operatorName ):
        args = [ ]

        print( operatorName )
        for i in range( self.argCount ):
            args.append( argumentGenerators[ self.argTypes[ i ] ]( ) )

        if args:
            print( args )
            return ' '.join( args ) + ' ' + operatorName
        else:
            return operatorName


#******************************************************************************
#
#  checkForVariable
#
#******************************************************************************

def checkForVariable( term ):
    if not isinstance( term, str ):
        return term

    # first check for a variable name or history expression
    if not term or term[ 0 ] != '$':
        return term

    if not g.interactive and term[ 1 : ] in g.userVariables:
        return g.userVariables[ term[ 1 : ] ]

    return RPNVariable( term[ 1 : ] )


#******************************************************************************
#
#  callers
#
#******************************************************************************

callers = [
    lambda func, args: func( ),
    lambda func, arg: func( arg ),
    lambda func, arg1, arg2: func( arg1, arg2 ),

    # 3, 4, and 5 argument functions don't recurse with lists more than one level
    # I have some ideas about how to improve this.
    lambda func, arg1, arg2, arg3:
        [ func( a, b, c ) for a in arg1 for b in arg2 for c in arg3 ],
    lambda func, arg1, arg2, arg3, arg4:
        [ func( a, b, c, d ) for a in arg1 for b in arg2 for c in arg3 for d in arg4 ],
    lambda func, arg1, arg2, arg3, arg4, arg5:
        [ func( a, b, c, d, e ) for a in arg1 for b in arg2 for c in arg3 for d in arg4 for e in arg5 ],
]


#******************************************************************************
#
#  sideEffectOperators
#
#  This is a list of operators that execute without modifying the result
#  stack.
#
#******************************************************************************

sideEffectOperators = [
    'comma_mode',
    'hex_mode',
    'identify_mode',
    'leading_zero_mode',
    'octal_mode',
    'timer_mode',
]

